import "./App.css";
import { VALUE } from "../Button/Button";
import { Board } from "../Board/Board";
import { useState } from "react";

export const MESSAGE = {
  IN_GAME: 0,
  WIN: 1,
  DRAW: 2,
};

function App() {
  const [nextValue, setNextValue] = useState(VALUE.X);
  const [winner, setWinner] = useState(null);
  const [values, setValues] = useState(Array(9).fill(VALUE.EMPTY));
  const inGameMessage = ["Turn of ", nextValue];
  const drawMessage = "It's a draw";
  const winMessage = ["The winner is ", winner];
  const message = winner
    ? winner === "Draw"
      ? drawMessage
      : winMessage
    : inGameMessage;

  function handleClick(i) {
    if (!winner) {
      if (values[i] === VALUE.EMPTY) {
        let newValues = values.slice();
        newValues[i] = nextValue;
        setValues(newValues);
        let winner = calculateWinner(newValues);
        if (winner) {
          setWinner(winner);
        } else if (newValues.filter((v) => v === VALUE.EMPTY).length === 0) {
          setWinner("Draw");
        }
        alternateNextValue();
      }
    }
  }

  function newGame() {
    setNextValue(VALUE.X);
    setWinner(null);
    setValues(Array(9).fill(VALUE.EMPTY));
  }

  function alternateNextValue() {
    setNextValue(nextValue === VALUE.X ? VALUE.O : VALUE.X);
  }

  return (
    <div className="center">
      <h1>{message}</h1>
      <Board nextValue={nextValue} handleClick={handleClick} values={values} />
      <button className="refresh-button" onClick={newGame}>
        <i className="icon-refresh"></i>
      </button>
    </div>
  );
}

function calculateWinner(values) {
  const lines = [
    [0, 1, 2],
    [3, 4, 5],
    [6, 7, 8],
    [0, 3, 6],
    [1, 4, 7],
    [2, 5, 8],
    [0, 4, 8],
    [2, 4, 6],
  ];

  let line = lines.find(
    ([a, b, c]) =>
      values[a] === values[b] &&
      values[b] === values[c] &&
      values[a] !== VALUE.EMPTY
  );
  let winner = line ? values[line[0]] : null;
  return winner;
}

export default App;
